* UNIVERISAD NACIONAL AUTNOMA DE MXICO *
* COORDINACIN DE HUMANIDADES *
* PROGRAMA UNIVERSITARIO DE ESTUDIOS DEL DESARROLLO *

* Replication code for "An entropy-based approach to measure diet adherence", with *
* Curtis Huffman, Ana G. Ortega-Avila and Hctor E. Njera-Cataln *
* email: chuffman@unam.mx *

clear all

* Set directories *

gl data="[...]"
gl bases="[...]"

* Data from the Mexican National Survey of Household Income and Expenditure 2018 *
* Publicly available at https://www.inegi.org.mx/programas/enigh/nc/2018/ *

use "$data\gastoshogar.dta", clear

sort folioviv foliohog clave

*************************************
* FOOD-GROUP DISTRIBUTION OF CNA-DF *
* AS PER ENIGH 2018					*
*************************************

/* ENIGH18 EXPENDITURE CATALOG*/

di in red "FOOD GROUPS IN ENIG AS PER CNA-DF 2015"

gen     rubros2=1   if  (clave>="A001" & clave<="A024") | clave=="A122" /*Reubicamos elote */
replace rubros2=2   if  (clave>="A101" & clave<="A106" & clave!="A103") /*Reubicamos rbano*/
replace rubros2=3   if  (clave>="A137" & clave<="A143") | clave=="A146" /*Soya a leguminosas*/
replace rubros2=4   if  (clave>="A144" & clave<="A145")
 
replace rubros2=5   if  (clave>="A107" & clave<="A136" & clave!="A108" & clave!="A122") | clave=="A103" /*Reubicamos aguacate elote y rbano*/ /*sacamos cilantro, perejil y ajo*/
replace rubros2=6   if  (clave>="A147" & clave<="A172") 
replace rubros2=7   if  (clave>="A057" & clave<="A061") | (clave>="A063" & clave<="A065") | ///
					    (clave>="A025" & clave<="A037") | (clave>="A038" & clave<="A046") 
replace rubros2=8   if  (clave>="A047" & clave<="A056") | (clave=="A062") 
replace rubros2=9   if  (clave>="A066" & clave<="A067") | (clave>="A068" & clave<="A069") | (clave>="A070" & clave<="A071") | ///
					    (clave>="A072" & clave<="A074") 
replace rubros2=10   if  (clave>="A082" & clave<="A088") | (clave>="A091" & clave<="A092") /*Se mueve crema y mantequilla de lcteos a grasas*/

replace rubros2=11   if  (clave>="A075" & clave<="A081") 

replace rubros2=12   if  (clave>="A093" & clave<="A094") 

replace rubros2=13  if  (clave>="A095" & clave<="A096") | (clave>="A097" & clave<="A100") | (clave>="A089" & clave<="A090") | clave=="A108" /*Crema, mantequilla y aguacate a grasas*/				   

replace rubros2=14  if  (clave>="A173" & clave<="A175")


label define rubros2 ////
1 "Cereales " ////
2 "Tubrculos " ////
3 "Leguminosas " ////
4 "Oleaginosas " ////
5 "Verduras " ////
6 "Frutas " ////
7 "Carnes " ////
8 "Carnes procesadas " ////
9 "Pescados " ////
10 "Lcteos " ////
11 "Leche " ////
12 "Huevo " ////
13 "Aceites y grasas " ////
14 "Azcar y mieles"

label value rubros2 rubros2

* Keeping only Mexico City's food-group data *

keep if (clave>="A001" & clave<="A175") & real(substr(folioviv,1,2))==9 & //// 
clave!="A109" /*Ajo*/ & ////
clave!="A010" /*Galletas dulces*/ & ////
clave!="A014" /*Pan dulce empaquetado*/ & ////
clave!="A016" /*Pasteles y pastelillos en piezas o a granel, pastel casero*/ & ////
clave!="A017" /*Pasteles y pastelillos empaquetados*/ & ////
clave!="A022" /*Botanas: frituras, palomitas, cheetos, doritos etctera (excepto papas)*/ & ////
clave!="A023" /*Sopas instantneas*/ & ////
clave!="A105" /*Harina para pur de papa*/ & ////
clave!="A106" /*Papas fritas en bolsa o a granel*/ & ////
clave!="A119" /*Cilantro*/ &  ////
clave!="A128" /*Perejil*/ & ////
clave!="A171" /*Frutas en almbar y conserva*/ & ////
clave!="A172" /*Frutas cristalizadas, enchiladas y secas*/

******************************************
* CDMX Empirical Food Baskets estimation *
******************************************

*************************************************
* WEIGHT (ml or gr) DISTRIBUTION PER FOOD-GROUP *
*************************************************

collapse (sum) cantidad, by(folioviv foliohog rubros2)

bys folioviv foliohog: egen tot_weight=total(cantidad) 
gen emp_dist=cantidad/tot_weight

duplicates report folioviv foliohog rubros2

save "$bases\emp_dist.dta", replace

************************************************************************************
* CDMX Normative Food Baskets estimation 										   *
* REFERENCE 																	   *
* vila Curiel, Abelardo (2012), Construccin de una Canasta Normativa Alimentaria *
* para el DF, EVALA DF, Mxico DF. Available at								   *
* https://evalua.cdmx.gob.mx/storage/app/media/uploaded-files/files/Atribuciones/  *
*	      medicion-de-la-pobreza/3_cna_construccion.pdf							   *
************************************************************************************

use "$data\concentradohogar.dta", clear
merge 1:m folioviv foliohog using "$data\poblacion.dta"
drop _merge
sort folioviv foliohog parentesco

* Restricting the sample to Mexico City *

keep if real(substr(folioviv,1,2))==9 /*Slo CDMX*/

gen parent=substr(parentesco,1,1)
destring parent, replace

* Dropping non-regular residents in the household*

drop if parent==4 /*Trabajadores domsticos*/
drop if parent==7 /*Huspedes*/

destring edad sexo, replace

gen req_cal = . 

replace req_cal = cond(sexo==1, 659.1  , 605.7  ) if edad==0
replace req_cal = cond(sexo==1, 890.1  , 827.8  ) if edad==1
replace req_cal = cond(sexo==1, 1103.7 , 1050.3 ) if edad==2
replace req_cal = cond(sexo==1, 1308.7 , 1168.7 ) if edad==3
replace req_cal = cond(sexo==1, 1375.5 , 1233.9 ) if edad==4
replace req_cal = cond(sexo==1, 1443.6 , 1284.1 ) if edad==5
replace req_cal = cond(sexo==1, 1513.2 , 1336.3 ) if edad==6
replace req_cal = cond(sexo==1, 1582.6 , 1390.1 ) if edad==7
replace req_cal = cond(sexo==1, 1657   , 1452.8 ) if edad==8
replace req_cal = cond(sexo==1, 1742.1 , 1525   ) if edad==9
replace req_cal = cond(sexo==1, 1838.2 , 1606.7 ) if edad==10
replace req_cal = cond(sexo==1, 1954.7 , 1697.3 ) if edad==11
replace req_cal = cond(sexo==1, 2115.4 , 1784.8 ) if edad==12
replace req_cal = cond(sexo==1, 2314.6 , 1853.1 ) if edad==13
replace req_cal = cond(sexo==1, 2524.2 , 1891.2 ) if edad==14
replace req_cal = cond(sexo==1, 2677.2 , 1903.9 ) if edad==15
replace req_cal = cond(sexo==1, 2788   , 1947.2 ) if edad==16
replace req_cal = cond(sexo==1, 2841.5 , 1990.4 ) if edad==17
replace req_cal = cond(sexo==1, 2865.7 , 2033.6 ) if edad==18
replace req_cal = cond(sexo==1, 2817   , 2076.8 ) if edad==19
replace req_cal = cond(sexo==1, 2804.9 , 2067.3 ) if edad==20
replace req_cal = cond(sexo==1, 2792.8 , 2057.8 ) if edad==21
replace req_cal = cond(sexo==1, 2780.7 , 2048.4 ) if edad==22
replace req_cal = cond(sexo==1, 2768.6 , 2038.9 ) if edad==23
replace req_cal = cond(sexo==1, 2756.5 , 2029.4 ) if edad==24
replace req_cal = cond(sexo==1, 2744.4 , 2019.9 ) if edad==25
replace req_cal = cond(sexo==1, 2732.3 , 2010.4 ) if edad==26
replace req_cal = cond(sexo==1, 2720.2 , 2000.9 ) if edad==27
replace req_cal = cond(sexo==1, 2708.1 , 1991.4 ) if edad==28
replace req_cal = cond(sexo==1, 2696   , 1982   ) if edad==29
replace req_cal = cond(sexo==1, 2683.9 , 1972.5 ) if edad==30
replace req_cal = cond(sexo==1, 2671.9 , 1963   ) if edad==31
replace req_cal = cond(sexo==1, 2659.8 , 1953.5 ) if edad==32
replace req_cal = cond(sexo==1, 2647.7 , 1944.1 ) if edad==33
replace req_cal = cond(sexo==1, 2635.6 , 1934.6 ) if edad==34
replace req_cal = cond(sexo==1, 2623.5 , 1925.1 ) if edad==35
replace req_cal = cond(sexo==1, 2611.4 , 1915.6 ) if edad==36
replace req_cal = cond(sexo==1, 2599.3 , 1906.2 ) if edad==37
replace req_cal = cond(sexo==1, 2587.3 , 1896.7 ) if edad==38
replace req_cal = cond(sexo==1, 2575.2 , 1887.2 ) if edad==39
replace req_cal = cond(sexo==1, 2563.1 , 1877.8 ) if edad==40
replace req_cal = cond(sexo==1, 2551   , 1868.3 ) if edad==41
replace req_cal = cond(sexo==1, 2538.9 , 1858.9 ) if edad==42
replace req_cal = cond(sexo==1, 2526.9 , 1849.4 ) if edad==43
replace req_cal = cond(sexo==1, 2514.8 , 1839.9 ) if edad==44
replace req_cal = cond(sexo==1, 2502.7 , 1830.5 ) if edad==45
replace req_cal = cond(sexo==1, 2490.6 , 1821   ) if edad==46
replace req_cal = cond(sexo==1, 2478.6 , 1811.6 ) if edad==47
replace req_cal = cond(sexo==1, 2466.5 , 1802.1 ) if edad==48
replace req_cal = cond(sexo==1, 2454.4 , 1792.7 ) if edad==49
replace req_cal = cond(sexo==1, 2442.4 , 1783.2 ) if edad==50
replace req_cal = cond(sexo==1, 2430.3 , 1773.8 ) if edad==51
replace req_cal = cond(sexo==1, 2418.2 , 1764.3 ) if edad==52
replace req_cal = cond(sexo==1, 2406.2 , 1754.9 ) if edad==53
replace req_cal = cond(sexo==1, 2394.1 , 1745.4 ) if edad==54
replace req_cal = cond(sexo==1, 2382.1 , 1736   ) if edad==55
replace req_cal = cond(sexo==1, 2370   , 1726.5 ) if edad==56
replace req_cal = cond(sexo==1, 2357.9 , 1717.1 ) if edad==57
replace req_cal = cond(sexo==1, 2345.9 , 1707.7 ) if edad==58
replace req_cal = cond(sexo==1, 2333.8 , 1698.2 ) if edad==59
replace req_cal = cond(sexo==1, 2321.8 , 1688.8 ) if edad==60
replace req_cal = cond(sexo==1, 2309.7 , 1679.4 ) if edad==61
replace req_cal = cond(sexo==1, 2297.7 , 1669.9 ) if edad==62
replace req_cal = cond(sexo==1, 2285.6 , 1660.5 ) if edad==63
replace req_cal = cond(sexo==1, 2273.6 , 1651.1 ) if edad==64
replace req_cal = cond(sexo==1, 2261.5 , 1641.6 ) if edad==65
replace req_cal = cond(sexo==1, 2249.5 , 1632.2 ) if edad==66
replace req_cal = cond(sexo==1, 2237.4 , 1622.8 ) if edad==67
replace req_cal = cond(sexo==1, 2225.4 , 1613.4 ) if edad==68
replace req_cal = cond(sexo==1, 2213.2 , 1605.6 ) if edad==69
replace req_cal = cond(sexo==1, 2201.5 , 1597.1 ) if edad>=70

* NORMATIVE DISTRIBUTION PER FOOD-GROUP *

* Cereales 
gen     rubros2_1=.
replace rubros2_1= 0.0378 if edad==0 
replace rubros2_1= 0.1372 if edad>=1  & edad<=3 
replace rubros2_1= 0.1356 if edad>=4  & edad<=13 
replace rubros2_1= 0.1635 if edad>=14 & edad<=50 
replace rubros2_1= 0.1338 if edad>=51 & edad<=70
replace rubros2_1= 0.1250 if edad>=71 

* Tubrculos 
gen     rubros2_2=.
replace rubros2_2= 0.0070 if edad==0 
replace rubros2_2= 0.0283 if edad>=1  & edad<=3 
replace rubros2_2= 0.0234 if edad>=4  & edad<=13 
replace rubros2_2= 0.0343 if edad>=14 & edad<=50 
replace rubros2_2= 0.0366 if edad>=51 & edad<=70
replace rubros2_2= 0.0220 if edad>=71 

* Leguminosas  
gen     rubros2_3=.
replace rubros2_3= 0.012  if edad==0 
replace rubros2_3= 0.0191 if edad>=1  & edad<=3 
replace rubros2_3= 0.0281 if edad>=4  & edad<=13 
replace rubros2_3= 0.0335 if edad>=14 & edad<=50 
replace rubros2_3= 0.0268 if edad>=51 & edad<=70
replace rubros2_3= 0.0141 if edad>=71 

* Oleaginosas
gen     rubros2_4=.
replace rubros2_4= 0      if edad==0 
replace rubros2_4= 0.0020 if edad>=1  & edad<=3 
replace rubros2_4= 0.0030 if edad>=4  & edad<=13 
replace rubros2_4= 0.0048 if edad>=14 & edad<=50 
replace rubros2_4= 0.0067 if edad>=51 & edad<=70
replace rubros2_4= 0.0035 if edad>=71 
 
* Verduras
gen     rubros2_5=.
replace rubros2_5= 0.0776 if edad==0 
replace rubros2_5= 0.2106 if edad>=1  & edad<=3 
replace rubros2_5= 0.1781 if edad>=4  & edad<=13 
replace rubros2_5= 0.2386 if edad>=14 & edad<=50 
replace rubros2_5= 0.2177 if edad>=51 & edad<=70
replace rubros2_5= 0.2120 if edad>=71 

* Frutas
gen     rubros2_6=.
replace rubros2_6= 0.1085 if edad==0 
replace rubros2_6= 0.1707 if edad>=1  & edad<=3 
replace rubros2_6= 0.2269 if edad>=4  & edad<=13 
replace rubros2_6= 0.2078 if edad>=14 & edad<=50 
replace rubros2_6= 0.2177 if edad>=51 & edad<=70
replace rubros2_6= 0.2662 if edad>=71 

* Carnes
gen     rubros2_7=.
replace rubros2_7= 0.0418 if edad==0 
replace rubros2_7= 0.0332 if edad>=1  & edad<=3 
replace rubros2_7= 0.0318 if edad>=4  & edad<=13 
replace rubros2_7= 0.0428 if edad>=14 & edad<=50 
replace rubros2_7= 0.0381 if edad>=51 & edad<=70
replace rubros2_7= 0.0308 if edad>=71 

* Carnes procesadas
gen     rubros2_8=.
replace rubros2_8= 0      if edad==0 
replace rubros2_8= 0	  if edad>=1  & edad<=3 
replace rubros2_8= 0.0042 if edad>=4  & edad<=13 
replace rubros2_8= 0.0036 if edad>=14 & edad<=50 
replace rubros2_8= 0.0040 if edad>=51 & edad<=70
replace rubros2_8= 0	  if edad>=71 

* Pescados
gen     rubros2_9=.
replace rubros2_9= 0      if edad==0 
replace rubros2_9= 0.0166 if edad>=1  & edad<=3 
replace rubros2_9= 0.02   if edad>=4  & edad<=13 
replace rubros2_9= 0.0268 if edad>=14 & edad<=50 
replace rubros2_9= 0.0205 if edad>=51 & edad<=70
replace rubros2_9= 0.0132 if edad>=71 

* Lcteos
gen     rubros2_10=.
replace rubros2_10= 0.0090 if edad==0 
replace rubros2_10= 0.0308 if edad>=1  & edad<=3 
replace rubros2_10= 0.0750 if edad>=4  & edad<=13 
replace rubros2_10= 0.0285 if edad>=14 & edad<=50 
replace rubros2_10= 0.0713 if edad>=51 & edad<=70
replace rubros2_10= 0.0616 if edad>=71   									

* Leche
gen     rubros2_11=.
replace rubros2_11= 0.6972 if edad==0 
replace rubros2_11= 0.2925 if edad>=1  & edad<=3 
replace rubros2_11= 0.2343 if edad>=4  & edad<=13 
replace rubros2_11= 0.1675 if edad>=14 & edad<=50 
replace rubros2_11= 0.1785 if edad>=51 & edad<=70
replace rubros2_11= 0.2111 if edad>=71  

* Huevo
gen     rubros2_12=.
replace rubros2_12= 0.0060 if edad==0 
replace rubros2_12= 0.0224 if edad>=1  & edad<=3 
replace rubros2_12= 0.0117 if edad>=4  & edad<=13 
replace rubros2_12= 0.0109 if edad>=14 & edad<=50 
replace rubros2_12= 0.0116 if edad>=51 & edad<=70
replace rubros2_12= 0.0114 if edad>=71    

* Aceites y grasas
gen     rubros2_13=.
replace rubros2_13= 0.0030 if edad==0 
replace rubros2_13= 0.0224 if edad>=1  & edad<=3 
replace rubros2_13= 0.0188 if edad>=4  & edad<=13 
replace rubros2_13= 0.0252 if edad>=14 & edad<=50 
replace rubros2_13= 0.0253 if edad>=51 & edad<=70
replace rubros2_13= 0.0229 if edad>=71  

* Azcar y mieles
gen     rubros2_14=.
replace rubros2_14= 0	   if edad==0 
replace rubros2_14= 0.0146 if edad>=1  & edad<=3 
replace rubros2_14= 0.0094 if edad>=4  & edad<=13 
replace rubros2_14= 0.0126 if edad>=14 & edad<=50 
replace rubros2_14= 0.0112 if edad>=51 & edad<=70
replace rubros2_14= 0.0066 if edad>=71    

* WEIGHT OF INDIVIDUAL BASKETS AS PER ITS KCal DENSITY *
gen denkcal=.
replace denkcal= req_cal/0.780 if edad==0
replace denkcal= req_cal/1.072 if edad>=1   & edad<=3
replace denkcal= req_cal/1.051 if edad>=4   & edad<=13
replace denkcal= req_cal/1.156 if edad>=14  & edad<=50
replace denkcal= req_cal/1.078 if edad>=51  & edad<=70
replace denkcal= req_cal/0.940 if edad>=71


collapse (mean) rubros* [aw=denkcal], by(folioviv foliohog)
save "$bases\norm_dist.dta", replace

*****************************************************************
* Measuring Divergences with respect the Normative Distribution *
*****************************************************************

* Households *

use "$bases\norm_dist.dta", clear
*summ rubros2*
reshape long rubros2_, i(folioviv foliohog) j(rubros2)
ren rubros2_ norm_dist

merge 1:1 folioviv foliohog rubros2 using "$bases\emp_dist.dta"
drop _merge

label values rubros2 rubros2

merge m:1 folioviv foliohog using "$data\concentradohogar.dta", keepusing(factor tot_integ ubica_geo) keep(match)
drop _merge cantidad tot_weight

/* Divergence measures proposed by 
Magdalou, B., & Nock, R. (2011). Income distributions and decomposable 
divergence measures. Journal of Economic Theory, 146(6), 2440-2454.

D(ye||yr)=sum[phi(ye)-phi(yr)-(ye-yr)*dphi(yr)]
alpha==0, phi(c)=-ln(c), then D(ye||yr)= ye/yr-ln(ye/yr)-1
and 
alpha==1, phi(c)=c*ln(c),then D(ye||yr)=ye*ln(ye/yr)+yr-ye

0*log(0) is defined to be 0, thus p log (p/q) is defined to be 0 if p = 0

WARNING this also means that it is +infinite if p > q = 0. 
For some 70 (meh) households q=0 for processed meats (rubros2==6)
*/

replace emp_dist=0 if emp_dist==.
*Replacing q=0 with the minimum of q (.07%)
summ norm_dist if norm_dist!=0
replace norm_dist=r(min) if norm_dist==0

*pi ln(pi) is defined to be zero if pi = 0

*alpha==1, phi(c)=c*ln(c),then D(ye||yr)=ye*ln(ye/yr)+yr-ye

gen dalpha1= cond(emp_dist==0, norm_dist , emp_dist*ln(emp_dist/norm_dist)+norm_dist-emp_dist)

*alpha==0, phi(c)=-ln(c), then D(ye||yr)= ye/yr-ln(ye/yr)-1

*WARNING: RESTRICT LOGODDS FOR emp_dist==0 (way too many)

gen dalpha0=cond(-ln(emp_dist/norm_dist)>15, 14 , emp_dist/norm_dist-ln(emp_dist/norm_dist)-1)

egen sdalpha1=sum(dalpha1),  by(folioviv foliohog)

egen sdalpha0=sum(dalpha0),  by(folioviv foliohog)

* Normalizing FBQI [0,1], Household level*

gen sdalpha1n=1-exp(-sdalpha1)

* Mexico City

collapse (mean) dalpha1 sdalpha1 sdalpha1n emp_dist norm_dist [fw=factor] , by(rubros2)

* END OF FILE *
